"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayerInfo = exports.PlayerOutfit = exports.TaskState = void 0;
const constant_1 = require("@skeldjs/constant");
const roles_1 = require("../roles");
class TaskState {
    constructor(
    /**
     * The type of task that this task is, see {@link TaskType}. Not *necessarily*
     * available, although should be.
     */
    taskType, 
    /**
     * Whether or not this task has been completed or not by the player.
     */
    completed) {
        this.taskType = taskType;
        this.completed = completed;
    }
}
exports.TaskState = TaskState;
/**
 * Specifically represents a set of cosmetics for a player, this is used as a
 * layering system to also show what cosmetics the player if they have shapeshifted
 * into another player.
 */
class PlayerOutfit {
    constructor(outfitType, name, color, hatId, petId, skinId, visorId, nameplateId) {
        this.outfitType = outfitType;
        this.name = name;
        this.color = color;
        this.hatId = hatId;
        this.petId = petId;
        this.skinId = skinId;
        this.visorId = visorId;
        this.nameplateId = nameplateId;
    }
    get isIncomplete() {
        return this.name === ""
            || this.color === -1
            || this.hatId === "missing"
            || this.petId === "missing"
            || this.skinId === "missing"
            || this.visorId === "missing"
            || this.name === "missing";
    }
    static createDefault(outfitType) {
        return new PlayerOutfit(outfitType, "", -1, "missing", "missing", "missing", "missing", "missing");
    }
    static Deserialize(reader, type) {
        const name = reader.string();
        const color = reader.packed();
        const hatId = reader.string();
        const petId = reader.string();
        const skinId = reader.string();
        const visorId = reader.string();
        const nameplateId = reader.string();
        return new PlayerOutfit(type, name, color, hatId, petId, skinId, visorId, nameplateId);
    }
    Serialize(writer) {
        writer.string(this.name);
        writer.packed(this.color);
        writer.string(this.hatId);
        writer.string(this.petId);
        writer.string(this.skinId);
        writer.string(this.visorId);
        writer.string(this.nameplateId);
    }
    clone(outfitType) {
        return new PlayerOutfit(outfitType, this.name, this.color, this.hatId, this.petId, this.skinId, this.visorId, this.nameplateId);
    }
}
exports.PlayerOutfit = PlayerOutfit;
/**
 * Represents information about a player, including any cosmetics they have equipped,
 * whether they are dead or the impostor, their role and whatever tasks they have.
 */
class PlayerInfo {
    constructor(gamedata, playerId, 
    /**
     * A collection of this player's outfits as they should be displayed.
     * Somewhat of a layer system, the Default outfit shouldn't be displayed
     * if there is another outfit in this object.
     */
    outfits, 
    /**
     * This player's level/rank.
     */
    playerLevel, 
    /**
     * Any flags that this player has, see {@link PlayerDataFlags}.
     */
    flags = 0, 
    /**
     * Which player this role is. Note that this is not their actual instance
     * of this role, see {@link PlayerData.role}, but is instead the class
     * used to create the instance, and holds metadata about the role, such as
     * the team it is for and whether it's a role for ghosts or not.
     *
     * You can use this to know what class {@link PlayerData.role} will be an
     * instance of.
     * @example
     * ```ts
     * if (player.playerInfo.roleType === ImpostorRole) {
     *   player.role instanceof ImpostorRole // true
     * }
     * ```
     */
    roleType, 
    /**
     * All of this player's tasks, and whether or not they have been completed
     * or not by the player.
     */
    taskStates = []) {
        this.gamedata = gamedata;
        this.playerId = playerId;
        this.outfits = outfits;
        this.playerLevel = playerLevel;
        this.flags = flags;
        this.roleType = roleType;
        this.taskStates = taskStates;
        this.currentOutfitType = constant_1.PlayerOutfitType.Default;
    }
    /**
     * Create a default player info object.
     * @param gamedata The gamedata object that this player information belongs to.
     * @param playerId The ID of the player.
     * @returns A default player info object.
     */
    static createDefault(gamedata, playerId) {
        return new PlayerInfo(gamedata, playerId, {
            [constant_1.PlayerOutfitType.Default]: PlayerOutfit.createDefault(constant_1.PlayerOutfitType.Default)
        }, 0, 0, roles_1.CrewmateRole, []);
    }
    static Deserialize(reader, gamedata, playerId) {
        const player = this.createDefault(gamedata, playerId);
        player.Deserialize(reader);
        return player;
    }
    /**
     * The player that this info is for.
     */
    getPlayer() {
        return this.gamedata.room.getPlayerByPlayerId(this.playerId);
    }
    /**
     * Whether this player has been flagged as disconnected.
     */
    get isDisconnected() {
        return (this.flags & constant_1.PlayerDataFlags.IsDisconnected)
            === constant_1.PlayerDataFlags.IsDisconnected;
    }
    /**
     * Whether this player has been flagged as the impostor.
     */
    get isImpostor() {
        return this.roleType.roleMetadata.roleTeam === constant_1.RoleTeamType.Impostor;
    }
    /**
     * Whether this player has been flagged as dead.
     */
    get isDead() {
        return (this.flags & constant_1.PlayerDataFlags.IsDead)
            === constant_1.PlayerDataFlags.IsDead;
    }
    get defaultOutfit() {
        return this.getOutfit(constant_1.PlayerOutfitType.Default);
    }
    get currentOutfit() {
        return this.getOutfit(this.currentOutfitType);
    }
    Deserialize(reader) {
        this.outfits = {};
        const numOutfits = reader.uint8();
        for (let i = 0; i < numOutfits; i++) {
            const outfitType = reader.uint8();
            this.outfits[outfitType] = reader.read(PlayerOutfit, outfitType);
        }
        this.playerLevel = reader.upacked();
        this.flags = reader.uint8();
        const roleType = reader.uint16();
        this.roleType = this.gamedata.room.registeredRoles.get(roleType) || roles_1.UnknownRole(roleType);
        this.taskStates = [];
        const numTasks = reader.uint8();
        for (let i = 0; i < numTasks; i++) {
            const taskIdx = reader.upacked();
            if (this.taskStates[taskIdx]) {
                this.taskStates[taskIdx].completed = reader.bool();
            }
            else {
                this.taskStates[taskIdx] = new TaskState(0, reader.bool());
            }
        }
    }
    Serialize(writer) {
        var _a;
        const outfitVals = Object.values(this.outfits);
        writer.uint8(outfitVals.length);
        for (const outfit of outfitVals) {
            writer.uint8(outfit.outfitType);
            writer.write(outfit);
        }
        writer.upacked(this.playerLevel);
        writer.uint8(this.flags);
        writer.uint16(this.roleType.roleMetadata.roleType);
        writer.uint8(((_a = this.taskStates) === null || _a === void 0 ? void 0 : _a.length) || 0);
        for (let i = 0; i < this.taskStates.length; i++) {
            writer.upacked(i);
            writer.bool(this.taskStates[i].completed);
        }
    }
    /**
     * Clone the player info for another player.
     */
    clone(playerId) {
        const clonedOutfits = {};
        const outfitEntries = Object.values(this.outfits);
        for (const outfit of outfitEntries) {
            clonedOutfits[outfit.outfitType] = new PlayerOutfit(outfit.outfitType, outfit.name, outfit.color, outfit.hatId, outfit.petId, outfit.skinId, outfit.visorId, outfit.nameplateId);
        }
        return new PlayerInfo(this.gamedata, playerId, clonedOutfits, this.playerLevel, this.flags, this.roleType, this.taskStates.map(task => new TaskState(task.taskType, task.completed)));
    }
    /**
     * Set whether this player is flagged as disconnected.
     */
    setDisconnected(isDisconnected) {
        if (isDisconnected) {
            this.setFlags(this.flags | constant_1.PlayerDataFlags.IsDisconnected);
        }
        else {
            this.setFlags(this.flags & ~constant_1.PlayerDataFlags.IsDisconnected);
        }
    }
    /**
     * Set whether this player is flagged as an impostor.
     */
    setImpostor(isImpostor) {
        if (isImpostor) {
            this.setFlags(this.flags | constant_1.PlayerDataFlags.IsImpostor);
        }
        else {
            this.setFlags(this.flags & ~constant_1.PlayerDataFlags.IsImpostor);
        }
    }
    /**
     * Set whether this player is flagged as dead.
     */
    setDead(isDead) {
        if (isDead) {
            this.setFlags(this.flags | constant_1.PlayerDataFlags.IsDead);
        }
        else {
            this.setFlags(this.flags & ~constant_1.PlayerDataFlags.IsDead);
        }
    }
    /**
     * Create or overwrite one of this player's outfits.
     * @param outfit The outfit to set.
     */
    setOutfit(outfit) {
        this.outfits[outfit.outfitType] = outfit;
        this.gamedata.markDirty(this);
        // todo outfit update events
    }
    /**
     * Get an outfit from this player, or create it from the player's default base
     * outfit if it doesn't exist, or create a new one entirely if that doesn't
     * exist.
     * @param outfitType The outfit to get.
     */
    getOutfit(outfitType) {
        const outfit = this.outfits[outfitType];
        if (outfit) {
            return outfit;
        }
        const defaultOutfit = this.outfits[constant_1.PlayerOutfitType.Default];
        const newOutfit = defaultOutfit
            ? defaultOutfit.clone(outfitType)
            : PlayerOutfit.createDefault(outfitType);
        this.outfits[outfitType] = newOutfit;
        return newOutfit;
    }
    /**
     * Delete an outfit on this player.
     * @param outfitType the outfit to delete.
     */
    deleteOutfit(outfitType) {
        delete this.outfits[outfitType];
        if (this.currentOutfitType === outfitType) {
            this.currentOutfitType = constant_1.PlayerOutfitType.Default;
        }
    }
    setName(outfitType, name) {
        const outfit = this.getOutfit(outfitType);
        outfit.name = name;
    }
    setColor(outfitType, color) {
        const outfit = this.getOutfit(outfitType);
        outfit.color = color;
    }
    setHat(outfitType, hatId) {
        const outfit = this.getOutfit(outfitType);
        outfit.hatId = hatId;
    }
    setPet(outfitType, petId) {
        const outfit = this.getOutfit(outfitType);
        outfit.petId = petId;
    }
    setSkin(outfitType, skinId) {
        const outfit = this.getOutfit(outfitType);
        outfit.skinId = skinId;
    }
    setVisor(outfitType, visorId) {
        const outfit = this.getOutfit(outfitType);
        outfit.visorId = visorId;
    }
    setNameplate(outfitType, nameplateId) {
        const outfit = this.getOutfit(outfitType);
        outfit.nameplateId = nameplateId;
    }
    /**
     * Set the flags of this player.
     */
    setFlags(flags) {
        this.flags = flags;
        this.gamedata.markDirty(this);
    }
    /**
     * Set the tasks of this player.
     * @param taskIds The IDs of each task.
     */
    setTaskIds(taskIds) {
        this.gamedata.setTasks(this, taskIds); // todo: player.settasks event
    }
    /**
     * Set the task states of this player.
     */
    setTaskStates(taskStates) {
        this.taskStates = taskStates;
        this.gamedata.markDirty(this);
    }
    /**
     * Mark a task of this player as completed.
     */
    completeTask(state) {
        state.completed = true;
        this.gamedata.markDirty(this);
    }
}
exports.PlayerInfo = PlayerInfo;
//# sourceMappingURL=PlayerInfo.js.map